CREATE TABLE 'volume' (
'rowid' INTEGER NOT NULL  DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'disk_rowid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'disk' ('rowid'),
'index' INTEGER NOT NULL  DEFAULT NULL,
'name' TEXT NOT NULL  DEFAULT 'NULL'
);

CREATE TABLE 'extent' (
'rowid' INTEGER NOT NULL  DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'node_blocknum' INTEGER DEFAULT NULL,
'volume_rowid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'volume' ('rowid'),
'cnid' INTEGER NOT NULL  DEFAULT NULL,
'offset' INTEGER NOT NULL  DEFAULT NULL,
'length' INTEGER NOT NULL  DEFAULT NULL,
'blocknum' INTEGER NOT NULL  DEFAULT NULL,
'flags' INTEGER NOT NULL  DEFAULT NULL,
'crypto' INTEGER NOT NULL  DEFAULT NULL,
UNIQUE (cnid, offset, volume_rowid)
);

CREATE TABLE 'inode' (
'rowid' INTEGER NOT NULL  DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'node_blocknum' INTEGER NOT NULL  DEFAULT NULL,
'volume_rowid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'volume' ('rowid'),
'cnid' INTEGER NOT NULL  DEFAULT NULL,
'parent_cnid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'named' ('cnid'),
'extent_cnid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'extent' ('cnid'),
'created' DATETIME NOT NULL  DEFAULT 'NULL',
'modified' DATETIME NOT NULL  DEFAULT 'NULL',
'changed' DATETIME NOT NULL  DEFAULT 'NULL',
'accessed' DATETIME NOT NULL  DEFAULT 'NULL',
'flags' INTEGER NOT NULL  DEFAULT NULL,
'links' INTEGER NOT NULL  DEFAULT NULL,
'bsdflags' INTEGER NOT NULL  DEFAULT NULL,
'uid' INTEGER NOT NULL  DEFAULT NULL,
'gid' INTEGER NOT NULL  DEFAULT NULL,
'mode' INTEGER NOT NULL  DEFAULT NULL,
'name' TEXT DEFAULT NULL,
'datalen' INTEGER DEFAULT NULL,
UNIQUE (volume_rowid, cnid)
);

CREATE TABLE 'named' (
'rowid' INTEGER NOT NULL  DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'node_blocknum' INTEGER NOT NULL  DEFAULT NULL,
'volume_rowid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'volume' ('rowid'),
'cnid' INTEGER NOT NULL  DEFAULT NULL,
'name' TEXT NOT NULL  DEFAULT 'NULL',
'inode_cnid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'inode' ('cnid'),
'added' DATETIME NOT NULL  DEFAULT 'NULL',
'type' INTEGER NOT NULL  DEFAULT NULL,
UNIQUE (volume_rowid, cnid, name)
);

CREATE TABLE 'disk' (
'rowid' INTEGER NOT NULL  DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'info' TEXT DEFAULT NULL
);

CREATE TABLE 'xattr' (
'rowid' INTEGER DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'node_blocknum' INTEGER NOT NULL  DEFAULT NULL,
'volume_rowid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'volume' ('rowid'),
'cnid' INTEGER NOT NULL  DEFAULT NULL,
'name' TEXT NOT NULL  DEFAULT 'NULL',
'type' INTEGER NOT NULL  DEFAULT NULL /* 2: EA, 6: symlink, 17: rsrc fork */,
'data' TEXT DEFAULT NULL,
'loglen' INTEGER DEFAULT NULL,
'extent_cnid' INTEGER DEFAULT NULL REFERENCES 'extent' ('cnid'),
UNIQUE (volume_rowid, cnid, name)
);

CREATE TABLE 'log' (
'rowid' INTEGER DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'when' DATETIME NOT NULL  DEFAULT 'NULL',
'info' TEXT DEFAULT NULL
);

CREATE TABLE 'path' (
'rowid' INTEGER DEFAULT NULL PRIMARY KEY AUTOINCREMENT,
'volume_rowid' INTEGER NOT NULL  DEFAULT NULL REFERENCES 'volume' ('rowid'),
'cnid' INTEGER NOT NULL  DEFAULT NULL,
'path' TEXT NOT NULL  DEFAULT 'NULL',
UNIQUE (volume_rowid, cnid)
);

CREATE INDEX 'index_extent_cnid' ON 'extent' ('cnid');
CREATE INDEX 'index_inode_cnid' ON 'inode' ('cnid');
CREATE INDEX 'index_named_cnid' ON 'named' ('cnid');
CREATE INDEX 'index_xattr_cnid' ON 'xattr' ('cnid');
CREATE INDEX 'index_xattr_name' ON 'xattr' ('name');
CREATE INDEX 'index_path_cnid' ON 'path' ('cnid');
CREATE INDEX 'index_path_path' ON 'path' ('path');